/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is cookiebutton.
 *
 * The Initial Developer of the Original Code is
 * Pang Lih Wuei (basic@mozdev.org).
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var clearWhenDeny = false;
var tooltip;
var element;
var clearElement;
var browser;
var strings;
var cookie;
var prefService;
var consoleService;
var permissionManager;
var cookieManager;
var nsICookie;
var extPrefBranch;
var prefBranch;
var disableAttribute;

function disableElement()
{
	element.removeAttribute("value");
	element.setAttribute(disableAttribute, "true");
}

// get default premission from preferences
function getDefaultPref ()
{
	var behavior = 0;

	// for some earlier firebird builds
	//try { if(!prefBranch.getBoolPref("enable")) return "default-deny";	}
	//catch (e) { }
	//try { if(prefBranch.getBoolPref("enableForOriginatingWebsiteOnly")) behavior = 1; }
	//catch (e)
	//{
		// hopefully there will be no more changes to the cookie pref
		try { behavior = prefBranch.getIntPref("cookieBehavior"); }
		catch (e) {	consoleService.logStringMessage("cookiebutton could not read network.cookie.cookieBehavior pref"); }
	//}

	// cookies disabled
	if(behavior == 2) return "default-deny";

	// warn about cookies -- no longer in use
//			try { if(prefBranch.getBoolPref("warnAboutCookies")) return "default-warn"; }
//			catch (e) { dump("can't read network.cookie.warnAboutCookies");	}

	try
	{
		switch (prefBranch.getIntPref("lifetimePolicy"))
		{
		case 1: return "default-warn";
		case 2: return behavior == 0 ? "default-third-party-session" : "default-session";
		}
	}
	catch (e) { consoleService.logStringMessage("cookiebutton could not read network.cookie.lifetimePolicy pref");	}
	
	switch (behavior)
	{
	case 0: return "default-third-party"; // enable third party cookies
	case 1: return "default-allow";       // enable cookies
//			case 3:	return "default-p3p";         // base on p3p
	}
	return "default"; // should never happen
}

// check permission
function getPermission(location)
{
	switch(permissionManager.testPermission(location,"cookie"))
	{
	case permissionManager.ALLOW_ACTION: return "allow";
	case permissionManager.DENY_ACTION:  return "deny";
	// constant from nsCookiePermission.cpp
	// ALLOW_SESSION = 8
	case 8:                    return "allow-session";
	default:                   return getDefaultPref();
	}
}

// generate label from value and location
function labelGenerator (type,location)
{
	switch(type)
	{
		case "default-session":             return strings.getString("default.session");
		case "default-third-party-session": return strings.getString("default.3rdPartySession");
		case "default-third-party":         return strings.getString("default.3rdParty");
		case "default-allow":               return strings.getString("default.accept");
		case "default-deny":                return strings.getString("default.reject");
		case "default-warn":                return strings.getString("default.warn");
//				case "default-p3p":                 return "Default (Based on P3P settings)";
// on firefox 1.0.x at least, getFormattedString is buggy (only grabs the first character) so we use this slower replace hack
		case "allow-session":               return strings.getString("host.session").replace(/%s/,location.host);
		case "allow":                       return strings.getString("host.accept").replace(/%s/,location.host);
		case "deny":                        return strings.getString("host.reject").replace(/%s/,location.host);
	}
	return "";
}

function enableElement(location)
{
	value = getPermission(location);
	if (tooltip) tooltip.label = labelGenerator(value, location);
	element.setAttribute("value", value);
	element.removeAttribute(disableAttribute);
}

// update the button
function update ()
{
	var location = browser.currentURI;
	// check if we are http or https, if not disable button
	if(!location || (location.scheme.search(/^https?$/) == -1))
	{
		disableElement();
		return false;
	}
	enableElement(location);
	return true;
}

var prefObserver =
{
	observe        : function (subject, topic, data) {update();},
	QueryInterface : function (aIID) 
	{
		if (aIID.equals(Components.interfaces.nsIObserver) ||
			aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
			aIID.equals(Components.interfaces.nsISupports))
				return this;
		throw Components.results.NS_NOINTERFACE;
	}
};

function updateExtPref()
{
	clearWhenDeny = extPrefBranch.getBoolPref("clear-when-deny");
	if (extPrefBranch.getBoolPref("show-clear-cookies"))
		clearElement.removeAttribute("hidden");
	else
		clearElement.setAttribute("hidden", "true");
}

var extPrefObserver =
{
	observe        : function (subject, topic, data)
	{
		updateExtPref();
	},
	QueryInterface : function (aIID) 
	{
		if (aIID.equals(Components.interfaces.nsIObserver) ||
			aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
			aIID.equals(Components.interfaces.nsISupports))
				return this;
		throw Components.results.NS_NOINTERFACE;
	}
};

var locationListener =
{
	onLocationChange    : function (w, r, l)         {update(); return true;},
	// to avoid error messages and/or make firefox function properly, sigh, NOTIFY_LOCATION seems to notify more than location stuff
	onProgressChange    : function (w,r,cS,mS,cT,mT) {return true;},
	onSecurityChange    : function (w,r,s)           {return true;},
	onStateChange       : function (w,r,f,s)         {return true;},
	onStatusChange      : function (w,r,s,m)         {return true;},
	onLinkIconAvailable : function (href)            {return true;}

	// should not need update()s here, but we seem to be hitting a firebird bug where
	// onLocationChange sometimes fail to trigger
	// 2005: disabling, seems like the bug was fix
	//onProgressChange    : function (w,r,cS,mS,cT,mT) {update();return true;},
	//onSecurityChange    : function (w,r,s)           {update();return true;},
	//onStateChange       : function (w,r,f,s)         {update();return true;},
	//onStatusChange      : function (w,r,s,m)         {update();return true;},
	//onLinkIconAvailable : function (href)            {update();return true;}
};

function init (doc, atype, ele, tip, disableAttr)
{

	var classes = Components.classes;
	var ifaces  = Components.interfaces;
	clearElement      = doc.getElementById("cookie-perm-"+atype+"-clear");
	strings           = doc.getElementById("cookie-perm-"+atype+"-strings");
	browser           = doc.getElementById("content");
	element           = ele;
	tooltip           = tip;
	disableAttribute  = disableAttr;
	prefService       = classes["@mozilla.org/preferences;1"].getService(ifaces.nsIPrefService);
	consoleService    = classes['@mozilla.org/consoleservice;1'].getService(ifaces.nsIConsoleService);
	permissionManager = classes["@mozilla.org/permissionmanager;1"].getService(ifaces.nsIPermissionManager);
	cookieManager     = classes["@mozilla.org/cookiemanager;1"].getService(ifaces.nsICookieManager);
	nsICookie         = ifaces.nsICookie;
	extPrefBranch     = prefService.getBranch("extensions.cookiebutton.").QueryInterface(ifaces.nsIPrefBranch).QueryInterface(ifaces.nsIPrefBranchInternal);
	prefBranch        = prefService.getBranch("network.cookie.").QueryInterface(ifaces.nsIPrefBranch).QueryInterface(ifaces.nsIPrefBranchInternal);


	// observe cookie prefs
	prefBranch.addObserver("cookieBehavior", prefObserver,true);
// no longer used
//			prefBranch.addObserver("warnAboutCookies", prefObserver,true);

	// observe life time policy of cookies
	prefBranch.addObserver("lifetimePolicy", prefObserver,true);

	extPrefBranch.addObserver("clear-when-deny", extPrefObserver, true);
	extPrefBranch.addObserver("show-clear-cookies", extPrefObserver, true);
	
	// setup browser location webProgressListener
	browser.addProgressListener(locationListener,ifaces.nsIWebProgress.NOTIFY_LOCATION);

	updateExtPref();
	update();
}

// set permission
function setPermission (option)
{
	var location = browser.currentURI;
	permissionManager.remove(location.host,"cookie");
	switch(option)
	{
	case "allow-session": permissionManager.add(location,"cookie",8);   break;
	case "allow":         permissionManager.add(location,"cookie",permissionManager.ALLOW_ACTION); break;
	case "deny":
		permissionManager.add(location,"cookie",permissionManager.DENY_ACTION);
		// fall through
	case "default-deny":
		if (clearWhenDeny) clearCookies();
		break;
	}
	update();
}

function clearCookies ()
{
	var host = browser.currentURI.host;
	var iter = cookieManager.enumerator;
	var nsICookie = nsICookie;
	while(iter.hasMoreElements())
	{
		var cookie = iter.getNext().QueryInterface(nsICookie);
		if (host == cookie.host) cookieManager.remove(cookie.host, cookie.name, cookie.path, false);
	}
//		consoleService.logStringMessage("cleared cookies for " + host);
}

function handleMenuCommand (menuitem)
{
	if (menuitem.value != '')
		setPermission(menuitem.value);
	else
		clearCookies();
	return true;
}

// update dropdown menu
function updatePermMenu (items)
{
	// should not need this here, but we seem to be hitting a firebird bug where
	// the webProgressListener sometimes fail to trigger
	// 2005 : think this bug is fixed now.
	// if(!update()) return false;
	var location = browser.currentURI;

	var value = getPermission(location);
	var default_value = (value.substr(0,7)=="default") ? value : getDefaultPref();

	// "default" option
	items[0].value = default_value;

	// set popup/dropdown label/values
	var last = items.length - 1;
	for (var i = 0; i < last; i++)
	{

		var option = items[i].value;
		if (option == value) items[i].setAttribute("checked","true");
		items[i].label = labelGenerator(option,location);
	}
	// clear cookies menuitem
// on firefox 1.0.x at least, getFormattedString is buggy (only grabs the first character) so we use this slower replace hack
	items[last].label = strings.getString("host.clear").replace(/%s/,location.host);
	return true;
}

// disable tooltip and populate menu items
function menupopupshowing(menu)
{
	if (tooltip) tooltip._cookieEnabled = false;
	return updatePermMenu(menu.getElementsByTagName("menuitem"));
}

// re-enable tooltip
function menupopuphiding()
{
	if (tooltip) tooltip._cookieEnabled = true;
	return true;
}

function getObject()
{
	var obj = {};
	obj.command = handleMenuCommand;
	obj.menupopupshowing = menupopupshowing;
	obj.menupopuphiding = menupopuphiding;
	return obj;
}